@@ -14,6 +14,7 @@ from __future__ import absolute_import
 import edflib
 import nsim
 import numpy as np
+from os import path
 
 def timeseries_from_edf(filename):
     """load a multi-channel Timeseries from an EDF (European Data Format) file
@@ -24,6 +25,8 @@ def timeseries_from_edf(filename):
     Returns: 
       Timeseries
     """
+    if not path.isfile(filename):
+        raise nsim.Error("file not found: '%s'" % filename)
     e = edflib.EDF(filename)
     if np.ptp(e.signal_nsamples) != 0:
         raise nsim.Error('channels have differing numbers of samples')
@@ -34,7 +37,7 @@ def timeseries_from_edf(filename):
     channelnames = e.signal_labels
     dt = 1.0/e.samplefreqs[0]
     ar = np.zeros((n, m), dtype=np.float64, order='F')
-    for i in xrange(m):
+    for i in range(m):
         e.edf.readsignal(i, 0, n, ar[:, i])
     tspan = np.arange(0, (n - 1 + 0.5) * dt, dt, dtype=np.float64)
-    return nsim.Timeseries(ar, tspan, channelnames)
+    return nsim.Timeseries(ar, tspan, labels=[None, channelnames])
