@@ -13,9 +13,19 @@ classes:
 
 from __future__ import absolute_import
 import numpy as np
+from sys import hexversion
+import warnings
+import copy
 import types
 import numbers
 
+# types for compatibility across python 2 and 3
+_SliceType = type(slice(None))
+_EllipsisType = type(Ellipsis)
+_TupleType = type(())
+
+warnings.filterwarnings('ignore', 'comparison to `None`', FutureWarning)
+
 
 class Timeseries(np.ndarray):
     """A numpy array with extra methods for applying time series analyses.
@@ -31,11 +41,11 @@ class Timeseries(np.ndarray):
             or repeated simulation.                  
 
     Thus the shape of a Timeseries array is (N, n, m) where N is the 
-    total number of time steps, n is the number of variables of a single node 
-    and m is the number of nodes.
+    total number of time points, n is the number of variables or channels of a
+    single node and m is the number of nodes.
 
-    Slice by sample index:    timeseries[103280:104800]
-    Slice by time in seconds: timeseries.t[10.0:20.0]
+    Slice by array index:  timeseries[103280:104800]
+    Slice by time in seconds:  timeseries.t[10.0:20.5]
 
     Methods:
       All functions defined in the package nsim.analyses1 are available as 
@@ -45,11 +55,31 @@ class Timeseries(np.ndarray):
       Timeseries.add_analyses('file.py')
     
     Attributes:
-      tspan: An array of shape (N,) defining the time in seconds that is 
-             meant by each time point. This should always remain sorted.
-      channelnames (array of str, optional): Name of each channel/variable
+      tspan: An array of shape (N,) giving the time in seconds at each 
+        time point along axis 0 of the array. tspan will always remain sorted.
+        The time points are not necessarily evenly spaced.
+
+      t: Allows slicing by time: timeseries.t[starttime:endtime, ...]
+
+      labels (list): For each axis>0, optionally gives names to label 
+        points along the axis. (labels[0] is always None as axis 0 is the
+        time axis, labeled by numbers in `tspan` rather than by strings) 
+        For i>0, labels[i] is a list of str of length shape(timeseries)[i]
+        giving names to each position along axis i. For example labels[1] can 
+        hold names for each variable or channel in a multivariate timeseries.
     """
-    def __new__(cls, input_array, tspan=None, fs=None, channelnames=None):
+    def __new__(cls, input_array, tspan=None, labels=None, fs=None):
+        """Create a new Timeseries from an ordinary numpy array.
+        Args:
+          input_array (ndarray): the timeseries data.
+          tspan (ndarray, optional): An array of shape (N,) giving the time 
+            in seconds at each time point along axis 0 of the array. tspan will
+            always remain sorted. Time points are not necessarily evenly spaced
+          labels (list, optional): For each axis>0, optionally give names 
+            to label points along the axis. (labels[0] must be None).
+          fs (scalar, optional): sample rate. Can be given instead of tspan if
+            all time points are evenly spaced.
+        """
         #print('In __new__')
         obj = np.asarray(input_array).view(cls)
         if tspan is not None:
@@ -61,21 +91,26 @@ class Timeseries(np.ndarray):
                 fs = 1.0
             n = len(input_array)
             obj.tspan = np.linspace(0.0, 1.0*(n-1)/fs, n, endpoint=True)
-        if isinstance(input_array, Timeseries) and channelnames is None:
-            obj.channelnames = input_array.channelnames
+        if isinstance(input_array, Timeseries) and labels is None:
+            obj.labels = input_array.labels
         else:
-            obj.channelnames = channelnames
-        if isinstance(channelnames, list): # TODO: clean this up
-            obj.channelnames = np.array(channelnames)
-        if obj.channelnames is None:
-            if input_array.ndim is 1:
-                obj.channelnames = np.array([''])
-            else:
-                obj.channelnames = np.array([''] * input_array.shape[1])
+            obj.labels = labels
+        if obj.labels is None:
+            obj.labels = [None] * obj.ndim
+        if len(obj.labels) < obj.ndim:
+            obj.labels.extend([None] * (obj.ndim - len(obj.labels)))
+        assert(obj.labels[0] is None) # time axis should not have string labels
+        for i, seq in enumerate(obj.labels):
+            if seq is not None:
+                if len(seq) != obj.shape[i]:
+                    raise ValueError(
+                        '%d labels given for axis %d of length %d' % (
+                        len(seq), i, obj.shape[i]))
+                obj.labels[i] = list(seq)
         obj.t = _Timeslice(obj)
         return obj
 
-    def __init__(self, input_array, tspan=None, channelnames=None):
+    def __init__(self, input_array, tspan=None, labels=None):
         #print('In __init__')
         pass
 
@@ -160,7 +195,8 @@ class Timeseries(np.ndarray):
         if isinstance(obj, self.__class__):
             if obj.shape is () or obj.shape is not () and len(self) ==len(obj):
                 self.tspan = obj.tspan
-                self.channelnames = obj.channelnames
+                self.labels = obj.labels
+                self.t = _Timeslice(self)
 
     def __array_prepare__(self, in_arr, context=None):
         #print('In __array_prepare__')
@@ -171,47 +207,64 @@ class Timeseries(np.ndarray):
         return super(Timeseries, self).__array_wrap__(out_arr, context)
 
     def __getitem__(self, index):
-        """When a Timeseries is sliced, tspan will also be sliced in the 
-        same way as axis 0 of the Timeseries.
-        channelnames will be sliced in the same way as axis 1.
-        If the resulting array is not a Timeseries then return an ndarray.
+        """When a Timeseries is sliced, tspan will be sliced in the 
+        same way as axis 0 of the Timeseries, and labels[i] will be sliced in 
+        the same way as axis i of the Timeseries.
+        If the resulting array is not a Timeseries (for example if the 
+        time axis is sliced to a scalar value) then returns an ndarray.
         """
-        #TODO add support for multidimensional array indexing
+        #TODO add support for using a multidimensional array as index
         #print('In __getitem__, index is ' + str(index))
         new_array = np.asarray(self).__getitem__(index)
+        cur_labels = self.labels
+        cur_shape = self.shape
         is_ts = True
         if (isinstance(index, numbers.Integral) or 
-                isinstance(index, types.SliceType) or
-                isinstance(index, types.EllipsisType) or 
+                isinstance(index, _SliceType) or
+                isinstance(index, _EllipsisType) or 
                 isinstance(index, np.ndarray) and index.ndim is 1):
             new_tspan = self.tspan[index]
-            new_channelnames = self.channelnames
-        elif isinstance(index, types.TupleType):
+            new_labels = cur_labels
+        elif isinstance(index, _TupleType):
+            if np.newaxis in index or Ellipsis in index:
+                index = copy.deepcopy(index)
             if Ellipsis in index:
                 pos = index.index(Ellipsis)
-                while len(index) < self.ndim:
+                while len(index) < self.ndim + index.count(np.newaxis):
                     index = index[:pos] + (slice(None),)*2 + index[(pos+1):]
+            while np.newaxis in index:
+                pos = index.index(np.newaxis)
+                if pos is 0:
+                    is_ts = False # prepending an axis: no longer a timeseries
+                index = index[:pos] + (slice(None),) + index[(pos+1):]
+                cur_labels = cur_labels[:pos] + [None] + cur_labels[pos:]
+                cur_shape = cur_shape[:pos] + (1,) + cur_shape[pos:]
             if self.tspan.shape is ():
-                new_tspan = None
-                new_channelnames = None
                 is_ts = False
             else:
                 new_tspan = self.tspan[index[0]]
-            if len(index) > 1:
-                new_channelnames = self.channelnames[index[1]]
-                if not isinstance(new_channelnames, np.ndarray):
-                    if new_array.ndim is 1:
-                        new_channelnames = np.array([''])
-                    else:
-                        new_channelnames = np.array([''] * new_array.shape[1])
+            if len(index) is 1:
+                new_labels = cur_labels
             else:
-                new_channelnames = self.channelnames
+                new_labels = [None] # labels[0] is always None
+                for i in range(1, len(index)):
+                    # using temp ndarray of labels ensures consistent slicing
+                    if cur_labels[i] is None:
+                        labelarray = np.array([''] * cur_shape[i])
+                    else:
+                        labelarray = np.array(cur_labels[i])
+                    newlabelarray = labelarray[index[i]]
+                    if newlabelarray.shape is not ():
+                        if cur_labels[i] is None:
+                            new_labels.append(None)
+                        else:
+                            new_labels.append(list(newlabelarray))
         elif index is None:
             is_ts = False
         if not isinstance(new_tspan, np.ndarray):
             is_ts = False
         if is_ts:
-            return Timeseries(new_array, new_tspan, new_channelnames)
+            return Timeseries(new_array, new_tspan, new_labels)
         else:
             return np.asarray(new_array)
 
@@ -236,16 +289,16 @@ class Timeseries(np.ndarray):
             last = self.tspan[-1]
         head = u'<%s of shape %s from time %f to %f>:\n' % (
             classname, self.shape, first, last)
-        repr_tspan = repr(self.tspan)
+        repr_tspan = 'tspan=' + repr(self.tspan)
         if len(repr_tspan) > 160:
-            repr_tspan = 'array([ %f, ..., %f ])' % (first, last)
+            repr_tspan = 'tspan=array([ %f, ..., %f ])' % (first, last)
         content = repr(np.asarray(self)).replace('array', 
-            classname, 1).rstrip(')') + ', \n        %s' % repr_tspan
-        if np.alltrue(self.channelnames == ''):
-            channames = ''
-        else: 
-            channames = ', \n        %s' % repr(self.channelnames)
-        return head + content + channames + ')'
+            classname, 1).rstrip(')') + ', \n' + repr_tspan
+        if all(l is None for l in self.labels):
+            labelsrep = ''
+        else:
+            labelsrep = ', \nlabels=' + repr(self.labels)
+        return head + content + labelsrep + ')'
 
     def __reduce__(self):
         """Support pickling Timeseries instances by saving __dict__"""
@@ -259,7 +312,7 @@ class Timeseries(np.ndarray):
         self.__dict__.update(tsstate[1])
 
     def angle(self, deg=0):
-        return Timeseries(np.angle(self, deg=deg), self.tspan)
+        return Timeseries(np.angle(self, deg=deg), self.tspan, self.labels)
 
     # Some of the usual array operations on Timeseries return a plain ndarray. 
     # This depends on whether a time axis is present in the result:
@@ -284,25 +337,33 @@ class Timeseries(np.ndarray):
 
     def reshape(self, newshape, order='C'):
         """If axis 0 is unaffected by the reshape, then returns a Timeseries,
-        otherwise returns an ndarray. See numpy.ndarray.reshape() for details.
+        otherwise returns an ndarray. Preserves labels of axis j only if all 
+        axes<=j are unaffected by the reshape.  
+        See ``numpy.ndarray.reshape()`` for more information
         """
         oldshape = self.shape
+        ar = np.asarray(self).reshape(newshape, order=order)
         if (newshape is -1 and len(oldshape) is 1 or
                 (isinstance(newshape, numbers.Integral) and 
                     newshape == oldshape[0]) or 
-                (isinstance(newshape, types.TupleType) and 
+                (isinstance(newshape, _TupleType) and 
                     (newshape[0] == oldshape[0] or 
                      (newshape[0] is -1 and np.array(oldshape[1:]).prod() == 
                                             np.array(newshape[1:]).prod())))):
             # then axis 0 is unaffected by the reshape
-            return super(Timeseries, self).reshape(newshape, order=order)
+            newlabels = [None] * ar.ndim
+            i = 1
+            while i < ar.ndim and ar.shape[i] == oldshape[i]:
+                newlabels[i] = self.labels[i]
+                i += 1
+            return Timeseries(ar, self.tspan, newlabels)
         else:
-            return np.asarray(self).reshape(newshape, order=order)
+            return ar
 
     def min(self, axis=None, out=None):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).min(axis, out)
         else:
             return super(Timeseries, self).min(axis, out)
@@ -310,7 +371,7 @@ class Timeseries(np.ndarray):
     def max(self, axis=None, out=None):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).max(axis, out)
         else:
             return super(Timeseries, self).max(axis, out)
@@ -318,7 +379,7 @@ class Timeseries(np.ndarray):
     def ptp(self, axis=None, out=None):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).ptp(axis, out)
         else:
             return super(Timeseries, self).ptp(axis, out)
@@ -326,7 +387,7 @@ class Timeseries(np.ndarray):
     def mean(self, axis=None, dtype=None, out=None):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).mean(axis, dtype, out)
         else:
             return super(Timeseries, self).mean(axis, dtype, out)
@@ -334,7 +395,7 @@ class Timeseries(np.ndarray):
     def std(self, axis=None, dtype=None, out=None, ddof=0):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).std(axis, dtype, out, ddof)
         else:
             return super(Timeseries, self).std(axis, dtype, out, ddof)
@@ -342,7 +403,7 @@ class Timeseries(np.ndarray):
     def var(self, axis=None, dtype=None, out=None, ddof=0):
         if (axis is 0 or 
                 self.ndim is 1 or 
-                isinstance(axis, types.TupleType) and 0 in axis):
+                isinstance(axis, _TupleType) and 0 in axis):
             return np.asarray(self).var(axis, dtype, out, ddof)
         else:
             return super(Timeseries, self).var(axis, dtype, out, ddof)
@@ -376,27 +437,39 @@ class _Timeslice(object):
         ts = self.ts
         dt = (ts.tspan[-1] - ts.tspan[0]) / (len(ts) - 1)
         if isinstance(index, numbers.Number):
+            if not (index >= ts.tspan[0] and index <= ts.tspan[-1]):
+                raise ValueError('time %s not in valid range %g to %g' % (
+                        index, ts.tspan[0], ts.tspan[-1]))
             newix = ts.tspan.searchsorted(index)
             return ts[newix]
-        elif isinstance(index, types.SliceType):
-            if index.step is None:
-                start, stop = ts.tspan.searchsorted([index.start, index.stop])
+        elif isinstance(index, _SliceType):
+            if index.start is None or index.start < ts.tspan[0]:
+                startt = ts.tspan[0]
+            else:
+                startt = index.start
+            if index.stop is None or index.stop > ts.tspan[-1]:
+                stopt = ts.tspan[-1] + dt/2
+            else:
+                stopt = index.stop
+            start, stop = ts.tspan.searchsorted([startt, stopt])
+            if index.step is None or index.step <= dt:
                 return ts[slice(start, stop, None)]
             else:
-                n = np.floor_divide(index.start - index.stop, index.step)
-                times = np.linspace(index.start, index.stop, n, endpoint=False)
+                times = np.arange(startt, stopt, index.step)
+                if times[-1] == stopt:
+                    times = times[:-1]
                 indices = ts.tspan.searchsorted(times)
                 if indices[-1] == len(ts.tspan):
                     indices = indices[:-1]
                 return ts[indices]
-        elif isinstance(index, types.EllipsisType) or index is None:
+        elif isinstance(index, _EllipsisType) or index is None:
             return ts[index]
         elif isinstance(index, np.ndarray) and index.ndim is 1:
             indices = ts.tspan.searchsorted(index)
             if indices[-1] == len(ts.tspan):
                 indices = indices[:-1]
             return ts[indices]
-        elif isinstance(index, types.TupleType):
+        elif isinstance(index, _TupleType):
             timeix = index[0]
             otherix = index[1:]
             if len(otherix) is 1:
@@ -410,12 +483,13 @@ class _Timeslice(object):
             raise TypeError("Time slicing can't handle that type of index yet")
 
     def __setitem__(self, index, value):
+        #TODO update logic to match __getitem__
         ts = self.ts
         dt = (ts.tspan[-1] - ts.tspan[0]) / (len(ts) - 1)
         if isinstance(index, numbers.Number):
             newix = ts.tspan.searchsorted(index)
             return ts.__setitem__(newix, value)
-        elif isinstance(index, types.SliceType):
+        elif isinstance(index, _SliceType):
             if index.step is None:
                 start, stop = ts.tspan.searchsorted(index.start, index.stop)
                 return ts.__setitem__(slice(start, stop, None), value)
@@ -426,14 +500,14 @@ class _Timeslice(object):
                 if indices[-1] == len(ts.tspan):
                     indices = indices[:-1]
                 return ts.__setitem__(indices, value)
-        elif isinstance(index, types.EllipsisType) or index is None:
+        elif isinstance(index, _EllipsisType) or index is None:
             return ts.__setitem__(index, value)
         elif isinstance(index, np.ndarray) and index.ndim is 1:
             indices = ts.tspan.searchsorted(index)
             if indices[-1] == len(ts.tspan):
                 indices = indices[:-1]
             return ts.__setitem__(indices, value)
-        elif isinstance(index, types.TupleType):
+        elif isinstance(index, _TupleType):
             timeix = index[0]
             ts = ts.t[timeix]
             otherix = index[1:]
